# coding=utf-8
# /usr/bin/env python

'''
Author: wenqiangw
Email: wenqiangw@opera.com
Date: 2020-04-24 15:40
Desc:
'''
from pynovice.rule_engine import RuleParser

def rule_pattern(_iterm):
    age = _iterm.get('age',0)
    sms_len = _iterm.get('sms_len',0)
    micro_loan_len = _iterm.get('micro_loan_len', 0)
    endBillsLen = _iterm.get('endBillsLen', 0)
    maxDueday = _iterm.get('maxDueday', 0)
    this_apply_dueday_predicted_score = _iterm.get('this_apply_dueday_predicted_score', 0)

    _pattern = [
        'OR',
            ['between#rule_10',age,18,55],
            ['>#rule_11',sms_len,50],
            ['<#rule_12',micro_loan_len,8],
            ['and#rule_40',
                ['>',endBillsLen,4],
                ['>',maxDueday,20],
                ['>',this_apply_dueday_predicted_score,20]
             ]
    ]
    return _pattern


if __name__ == '__main__':
    iterm = {'age': 30, 'sms_len': 70, 'micro_loan_len': 10, 'endBillsLen': 5, 'maxDueday': 20,
             'this_apply_dueday_predicted_score': 10}
    rule = rule_pattern(iterm)

    engine = RuleParser()
    out = engine.evaluate(_rule=rule)
    print(out)


# coding=utf-8
# /usr/bin/env python

'''
Author: wenqiangw
Email: wenqiangw@opera.com
Date: 2020-04-30 14:42
Desc:
    标准评分卡(lr)
    log(odds)=w*x
    score = self.score_offset + self.score_factor * np.log(odds)
'''

from pynovice.score_card.score_card import ScoreCardModel
from sklearn.linear_model import LogisticRegression

class LRModel(ScoreCardModel):
    '''
        用于预测某一条预处理过的特征向量得分的方法
    :return:
        score_label: 最终预测的评分
        score_feature: 各个特征的得分字典
    '''

    def __init__(self):
        super().__init__()
        self.model_ = None
        self.woe_score = None

    def train(self,df_field,df_label,eval=False):
        self.model_ = LogisticRegression()
        self.model_.fit(df_field, df_label)
        if eval:
            self.woe_score = self.transform(df_field)
            print(self.woe_score)

    def predict(self,x):
        proba = self.model_.predict_proba([x])
        score_label = self.probability_to_score(bad_prob=proba[0][-1])
        return score_label

    def predict_detail(self,x):
        coef_ = self.model_.coef_[0]
        intercept_ = self.model_.intercept_[0]

        base_score = round(self.score_offset + self.score_factor * intercept_,2)  # 基础分
        field_score = [round(self.score_factor*coef_[i]*xi,2) for i,xi in enumerate(x)]  # 特征的分值
        final_score = int(base_score+sum(field_score))
        return final_score, base_score, field_score

    def transform(self,df_field):
        '''
        计算评分卡模型详细分数信息
        :param df_field:
        :return:
        '''
        df_eval = df_field.drop_duplicates()
        _out = []
        base_score = None
        for i, iter in df_eval.iterrows():
            _, base_score, field_score = self.predict_detail(x=iter)
            # 变量评分
            df_iter = pd.DataFrame(iter)
            df_iter.columns = ['value_']
            df_iter['score_'] = field_score
            _out.append(df_iter)
        df_score = pd.concat(_out, axis=0)
        df_score.loc['_base'] = [None, base_score]
        df_score.sort_index(inplace=True)
        return df_score


if __name__ == '__main__':
    import pandas as pd
    df = pd.DataFrame([[1, 2, 3, 4, 5, 5, 5, 6, 8, 3, 2, 1, 5, 7],[10, 2, 3, 42, 534, 5, 53, 6, 83, 3, 42, 21, 25, 7], [1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1]]).T
    df.columns = ['field1', 'field2', 'label']
    df_fields, df_label = df[['field1','field2']], df['label']
    print(df_fields)
    lr = LRModel()
    lr.train(df_fields,df_label,eval=True)
    print(lr.predict_detail([2,8]))



# coding=utf-8
# /usr/bin/env python

'''
Author: wenqiangw
Email: wenqiangw@opera.com
Date: 2020-04-30 14:42
Desc:
    基于违约率的评分模型
    score = self.score_offset + self.score_factor * np.log(odds)
'''

from pynovice.score_card.score_card import ScoreCardModel
import numpy as np

class BaseModel(ScoreCardModel):
    '''
        用于预测某一条预处理过的特征向量得分的方法
    :return:
        score_label: 最终预测的评分
    '''

    def __init__(self,bad_target=1,eps=1e-4):
        super().__init__()
        self.bad_target = bad_target
        self.eps = eps
        # _out
        self.base_logodds = None
        self.fields_logodds = None

    def train(self,df_fields,df_label):
        '''
            通过训练集的odds来创建违约率模型
        :param df_fields:
        :param df_label:
        :return:
        '''
        df_fields.reset_index(drop=True, inplace=True)
        df_label.reset_index(drop=True, inplace=True)
        _bad_rate = 1.00* (df_label==self.bad_target).sum()/df_label.size
        self.base_logodds = max(_bad_rate,self.eps)/max(1-_bad_rate,self.eps)
        self.fields_logodds = []
        for iname in df_fields.columns:
            df_concat = pd.concat([df_fields[iname], df_label], axis=1)
            log_odds = df_concat.groupby(iname).agg(
                lambda x:1.00* np.log(max(sum(x==self.bad_target),self.eps)/max(sum(x!=self.bad_target),self.eps)))
            odds_dict = dict(zip(log_odds.index,log_odds.values[:,0]))
            self.fields_logodds.append(odds_dict)

    def predict_detail(self,x):
        base_score = round(self.score_offset, 2)  # 基础分
        field_score = []
        for i,v in enumerate(x):
            _logodds = self.fields_logodds[i].get(v,self.base_logodds)
            _score = round(self.score_factor * _logodds,2)  # 特征的分值
            field_score.append(_score)
        final_score = int(base_score+sum(field_score))
        return final_score, base_score, field_score


if __name__ == '__main__':
    import pandas as pd
    df = pd.DataFrame([[1, 2, 3, 4, 5, 5, 5, 6, 8, 3, 2, 1, 5, 7],[10, 2, 3, 42, 534, 5, 53, 6, 83, 3, 42, 21, 25, 7], [1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1]]).T
    df.columns = ['field1', 'field2', 'label']
    df_fields, df_label = df[['field1','field2']], df['label']

    base = BaseModel()
    base.train(df_fields,df_label)
    print(base.predict_detail([2,10]))



import setuptools
#from version import get_git_version

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
	name='pynovice',
	version='0.1.10',
	description='data mining for novice and expert',
	url='https://github.com/wqwangchn/novice',
	author='wqwangchn',
	author_email='wqwangchn@163.com',
	long_description=long_description,
	long_description_content_type="text/markdown",
	packages=setuptools.find_packages(),
	include_package_data = True,
	install_requires = ['wordninja>=2.0.0','bert4keras>=0.6.5','tensorflow==2.1.0','reverse_geocoder==1.5.1',
						'dill==0.3.1.1 ','pyecharts==1.6.2'],
	classifiers=[
	"Programming Language :: Python :: 3",
	"License :: OSI Approved :: MIT License",
	"Operating System :: OS Independent",
	],
	zip_safe=False
)
